package com.vmp.demo.api.qrcode;

import com.vmp.demo.DefaultSpringbootTest;
import com.vmp.demo.env.Environment;
import com.vmp.demo.utils.SignUtil;
import lombok.extern.slf4j.Slf4j;
import net.sf.json.JSONObject;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.client.RestTemplate;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Date;

@Slf4j
public class PreOrder extends DefaultSpringbootTest {

    @Autowired
    private RestTemplate restTemplate;
    
    @Test
    public void test() throws Exception {

        Environment environment = Environment.TEST;

        // Step 1: Prepare your request parameters
        JSONObject reqJsonObject = new JSONObject();
        reqJsonObject.put("service", "service.wechat.qrcode.PreOrder");
        reqJsonObject.put("user_confirm_key", environment.getUserKey());
        reqJsonObject.put("transaction_amount", "0.02");
        // reqJsonObject.put("transaction_amount", {{your-transaction-amount}});
        reqJsonObject.put("out_trade_no", String.valueOf(new Date().getTime()));
        // reqJsonObject.put("out_trade_no", {{your-out-trade-no}});
        reqJsonObject.put("paytype", "WeChat");
        reqJsonObject.put("buyertype", "android");
        reqJsonObject.put("subject", "测试带中文的subject");
        reqJsonObject.put("payment_type", "WECHATCN");
        reqJsonObject.put("fee_type", "HKD");
        // optional
        // reqJsonObject.put("notify_url", {{your-notify-url}});
        // reqJsonObject.put("tid", {{your-tid}});
        
        reqJsonObject.put("time", LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss")));
        reqJsonObject.put("sign", SignUtil.doSign(reqJsonObject, environment.getSecretCode()));

        String requestBody = reqJsonObject.toString();
        log.info("request body: \n{}", requestBody);

        // Step 2: Preparation before request
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<String> requestEntity = new HttpEntity<>(requestBody, headers);

        // Step 3：Send request
        ResponseEntity<String> responseEntity = restTemplate.postForEntity(
                environment.getUrl(), requestEntity, String.class);

        log.info("responseEntity: {}", responseEntity);

        // Assert.assertEquals(200, responseEntity.getStatusCodeValue());
        
        // Step 4：Verify response code
        if (200 == responseEntity.getStatusCodeValue()) {
            String responseBody = responseEntity.getBody();
            log.info("response body: \n{}", responseBody);

            JSONObject respJsonObject = JSONObject.fromObject(responseBody);
            String returnStatus = respJsonObject.getString("return_status");
            // Assert.assertEquals("00", returnStatus);

            // Step 5：Verify api return status
            if ("00".equals(returnStatus)) {
                String paramSign = (String) respJsonObject.remove("sign");
                String signCode = SignUtil.doSign(respJsonObject, environment.getSecretCode());
                // Assert.assertEquals(paramSign, signCode);

                // Step 6：Verify sign
                if (paramSign.equals(signCode)) {
                    // do something
                    log.info("qr_code: {}", respJsonObject.getString("qr_code"));
                }
                else {
                    // do something
                    // Assert.fail("check sign fail");
                }
            }
            else {
                // do something
                // Assert.fail("returnStatus is not 00");
            }
        }
        else {
            // do something
            // Assert.fail("StatusCode is not 200");
        }
        

    }
    
}
